<?php

class PEAR_Autoloader
{
    private $autoloadMap = [];
    private $methodMap = [];

    public function addAutoload($method, $classname = null)
    {
        if (is_array($method)) {
            $method = array_change_key_case($method, CASE_LOWER);
            $this->autoloadMap = array_merge($this->autoloadMap, $method);
        } else {
            $this->autoloadMap[strtolower($method)] = $classname;
        }
    }

    public function removeAutoload($method)
    {
        $method = strtolower($method);
        $ok = isset($this->autoloadMap[$method]);
        unset($this->autoloadMap[$method]);
        return $ok;
    }

    public function addAggregateObject($classname)
    {
        $classname = strtolower($classname);
        if (!class_exists($classname)) {
            $includeFile = str_replace('\\', '/', $classname) . '.php';
            include_once $includeFile;
        }

        $obj = new $classname();
        $methods = get_class_methods($classname);
        
        foreach ($methods as $method) {
            if ($method[0] !== '_' && $method !== $classname) {
                $this->methodMap[$method] = $obj;
            }
        }
    }

    public function removeAggregateObject($classname)
    {
        $ok = false;
        $classname = strtolower($classname);

        foreach ($this->methodMap as $method => $obj) {
            if (is_a($obj, $classname)) {
                unset($this->methodMap[$method]);
                $ok = true;
            }
        }

        return $ok;
    }

    public function __call($method, $args)
    {
        $method = strtolower($method);
        
        if (empty($this->methodMap[$method]) && isset($this->autoloadMap[$method])) {
            $this->addAggregateObject($this->autoloadMap[$method]);
        }

        if (isset($this->methodMap[$method])) {
            return call_user_func_array([$this->methodMap[$method], $method], $args);
        }

        return false;
    }
}

?>
